open Util

let rec follow_link filename =
  try
    let new_filename =
      Filename.concat (Filename.dirname filename) (Unix.readlink filename) in
    follow_link new_filename
  with Unix.Unix_error _ -> filename

let env_search_path =
  try String.split ':' (Sys.getenv "SILCC_PATH")
  with Not_found ->
    let bin_name = follow_link (Sys.getenv "_") in
    let bin_dir =
      try Filename.dirname bin_name
      with Not_found ->
        prerr_endline "Please set SILCC_PATH.";
        exit 1 in
    let lib_dir = Filename.concat (Filename.dirname bin_dir) "lib" in
    [ lib_dir ]

let search_path = ref env_search_path

exception Unknown_module of string

let locate default_path filename =
  if Sys.file_exists (Filename.concat default_path filename) then
    Filename.concat default_path filename
  else
    try
      let p path =
        Sys.file_exists (Filename.concat path filename) in
      Filename.concat (List.find p !search_path) filename
    with Not_found -> raise (Unknown_module filename)

let rec module_filename module_path =
  match module_path with
    | [] -> failwith "module_filename"
    | [name] -> name ^ ".lcc"
    | head :: tail -> Filename.concat head (module_filename tail)

let current_default_path = ref "."

let queue = Queue.create ()

let reset () = Queue.clear queue

let is_empty () = Queue.is_empty queue

let add ?reference filename =
  Queue.add (reference, !current_default_path, filename) queue

let use_module (pos: Pos.range) module_path =
  let v = String.concat "." module_path in
  add ~reference:(pos, v) (module_filename module_path);
  v
